@@ -5,10 +5,9 @@ Miscellaneous functionality
 * **Search**: `nearest_element`, `nearest_unsorted`, `nearest_index`
 * **Sort**: `sort_natural`, `zipsort`, `issorted`
 * **Namedtuples**: `namedtupled_addcolumn`, `namedtuple_extend`, etc.
-* **IPython / Jupyter**: `inside_jupyter`, `is_interactive_session`, `session_type`
 * **Open files**: `open_with_standard_app`, `wait_for_file_modified`, `open_with`
 * **Unit conversions**: `cm_to_pixels`, `page_dinsize_to_mm`, etc.
-* **Other**: `runonce`, `singleton`
+* **Other**: `singleton`
 
 """
 # -*- coding: utf-8 -*-
@@ -20,15 +19,17 @@ from bisect import bisect as _bisect
 from collections import namedtuple as _namedtuple
 import re as _re
 import dataclasses
-import warnings
 
 import numpy as np
 from fractions import Fraction
 import numbers
+from .envir import inside_jupyter
+from .common import runonce
+
 
 from typing import TYPE_CHECKING
 if TYPE_CHECKING or 'sphinx' in _sys.modules:
-    from typing import TypeVar, Sequence, Union
+    from typing import TypeVar, Sequence, Union, Optional, Callable, Any, Iterable
     T = TypeVar("T")
     T2 = TypeVar("T2")
     number_t = Union[float, numbers.Rational]
@@ -255,7 +256,7 @@ def sort_natural(seq: list, key: Callable[[Any], str]=None) -> list:
     return sorted(seq, key=alphanum_key)
 
 
-def sort_natural_dict(d: Dict[str, Any], recursive=True) -> dict:
+def sort_natural_dict(d: dict[str, Any], recursive=True) -> dict:
     """
     sort dict d naturally and recursively
     """
@@ -272,7 +273,7 @@ def sort_natural_dict(d: Dict[str, Any], recursive=True) -> dict:
     return dict(sorted_rows)
 
 
-def issorted(seq:list, key=None) -> bool:
+def issorted(seq: list, key=None) -> bool:
     """
     Returns True if seq is sorted
 
@@ -788,7 +789,7 @@ def dictmerge(dict1: dict, dict2: dict) -> dict:
     return out
 
 
-def moses(pred: Callable[[T], bool], seq: Iterator[T]
+def moses(pred: Callable[[T], bool], seq: Iterable[T]
           ) -> tuple[list[T], list[T]]:
     """
     Divides *seq* into two lists: filter(pred, seq), filter(not pred, seq)
@@ -817,7 +818,7 @@ def moses(pred: Callable[[T], bool], seq: Iterator[T]
     return trueitems, falseitems
 
 
-def allequal(xs: Iterator) -> bool:
+def allequal(xs: Sequence) -> bool:
     """
     Return True if all elements in xs are equal
 
@@ -865,7 +866,6 @@ def snap_to_grid(x: number_t, tick: number_t, offset: number_t=0, nearest=True)
         the result will be float, if it is a Fraction, then the
         result will be a fraction
     """
-    assert isinstance(x, (float, number.Rational))
     t = x.__class__
     if nearest:
         return t(round((x - offset) / tick)) * tick + offset
@@ -874,7 +874,8 @@ def snap_to_grid(x: number_t, tick: number_t, offset: number_t=0, nearest=True)
 
 
 def snap_array(X: np.ndarray, tick:float, offset:float=0.,
-               out:Optional[np.ndarray]=None, nearest=True) -> np.ndarray:
+               out: Optional[np.ndarray] = None, nearest=True
+               ) -> np.ndarray:
     """
     Snap the values of X to the nearest slot in a grid
 
@@ -903,7 +904,11 @@ def snap_array(X: np.ndarray, tick:float, offset:float=0.,
     return _snap_array_floor(X, tick, offset=float(offset), out=out)
 
 
-def _snap_array_nearest(X: np.ndarray, tick: number_t, offset=0, out=None) -> np.ndarray:
+def _snap_array_nearest(X: np.ndarray, 
+                        tick: number_t, 
+                        offset: number_t = 0., 
+                        out: Optional[np.ndarray] = None
+                        ) -> np.ndarray:
     if out is None:
         out = X.copy()
     if offset != 0:
@@ -919,8 +924,8 @@ def _snap_array_nearest(X: np.ndarray, tick: number_t, offset=0, out=None) -> np
     return out
 
 
-def _snap_array_floor(X: np.ndarray, tick:float, offset=0., out: np.ndarray=None) -> \
-        np.ndarray:
+def _snap_array_floor(X: np.ndarray, tick:float, offset=0., out: np.ndarray=None
+                      ) -> np.ndarray:
     arr = out if out is not None else X.copy()
     if offset != 0:
 
@@ -937,7 +942,7 @@ def _snap_array_floor(X: np.ndarray, tick:float, offset=0., out: np.ndarray=None
 
 
 def snap_to_grids(x: number_t, ticks: Sequence[number_t], 
-                  offsets: Sequence[number_t]=None, mode='nearest'
+                  offsets: Sequence[number_t] = None, mode='nearest'
                   ) -> number_t:
     """
     Snap x to the nearest slot within multiple overlapping grids
@@ -1338,39 +1343,6 @@ def singleton(cls):
     return get_instance()
 
 
-class runonce:
-    """
-    To be used as decorator. `func` will run only once
-
-    Example::
-
-        # get_config() will only read the file the first time,
-        # return the resulting dict for any further calls
-
-        @runonce
-        def get_config():
-            config = json.load(open("/path/to/config.json"))
-            return config
-
-        config = get_config()
-
-    """
-    __slots__ = ('func', 'result', 'has_run')
-
-    def __init__(self, func):
-        self.func = func
-        self.result = None
-        self.has_run = False
-
-    def __call__(self, *args, **kwargs):
-        if self.has_run:
-            return self.result
-
-        self.result = self.func(*args, **kwargs)
-        self.has_run = True
-        return self.result
-
-
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 def istype(obj, *types) -> bool:
@@ -1542,7 +1514,8 @@ def open_with_app(path: str,
         _open_with_standard_app(path, wait=wait, min_wait=min_wait, timeout=timeout)
         return
 
-    import subprocess, time
+    import subprocess
+    import time
 
     if shell:
         assert isinstance(app, str), "shell needs a command-line as string"
@@ -1584,7 +1557,6 @@ def wait_for_file_modified(path:str, timeout:float=None) -> bool:
     """
     from watchdog.observers import Observer
     from watchdog.events import PatternMatchingEventHandler
-    import time
     directory, base = _os.path.split(path)
     if not directory:
         directory = "."
@@ -1627,81 +1599,10 @@ def first_existing_path(*paths: str, default="~") -> str:
     return _os.path.expanduser(default)
 
 
-@runonce
-def inside_jupyter() -> bool:
-    """
-    Are we running inside a jupyter notebook?
-    """
-    return session_type() == 'jupyter'
-
-@runonce
-def inside_ipython() -> bool:
-    """
-    Are we running inside ipython?
-
-    This includes any ipython session (ipython in terminal, jupyter, etc.)
-    """
-    return session_type() in ('jupyter', 'ipython', 'ipython-terminal')
-
-
-@runonce
-def is_interactive_session() -> bool:
-    """ Are we running inside an interactive session? """
-    return _sys.flags.interactive == 1
-
-
-@runonce
-def session_type() -> str:
-    """
-    Returns the kind of python session
-
-    .. note::
-        See also `is_interactive_session` to check if we are inside a REPL
-
-    Returns:
-        Returns one of "jupyter", "ipython-terminal" (if running ipython
-        in a terminal), "ipython" (if running ipython outside of a terminal),
-        "python" if running normal python.
-
-    """
-    try:
-        # get_ipython should be available within an ipython/jupyter session
-        shell = get_ipython().__class__.__name__   # type: ignore
-        if shell == 'ZMQInteractiveShell':
-            return "jupyter"
-        elif shell == 'TerminalInteractiveShell':
-            return "ipython-terminal"
-        else:
-            return "ipython"
-    except NameError:
-        return "python"
-
-
-@runonce
-def running_inside_terminal() -> bool:
-    """
-    Are we running inside a terminal and not in the background?
-
-    """
-    return _sys.stdin and _sys.stdin.isatty()
-
-
-def ipython_qt_eventloop_started() -> bool:
-    """
-    Are we running ipython / jupyter and the qt event loop has been started?
-    ( %gui qt )
-    """
-    session = session_type()
-    if session == 'ipython-terminal' or session == 'jupyter':
-        # we are inside ipython so we can just call 'get_ipython'
-        ip = get_ipython()   # type: ignore
-        return ip.active_eventloop == "qt"
-    else:
-        return False
-
-
-def html_table(rows: list, headers: list[str], maxwidths:Optional[list[int]]=None,
-               rowstyles:Optional[list[str]]=None) -> str:
+def html_table(rows: list, 
+               headers: list[str], 
+               maxwidths: Optional[list[int]]=None,
+               rowstyles: Optional[list[str]]=None) -> str:
     """
     Create a html table
 
